@@ -30,9 +30,9 @@ use Cwd qw(getcwd);
 
 my $cwd = getcwd();
 
-my $label    = "snapshot";     #  Automagically set to 'release' for releases.
-my $major    = "1";            #  Bump before release.
-my $minor    = "7.1";          #  Bump before release.
+my $label    = "snapshot";     #  Change this to 'release' just before making a release.
+my $major    = "1";            #  ...and this too.
+my $minor    = "5";            #  ...and this too.
 
 my $commits  = "0";
 my $hash1    = undef;          #  This from 'git describe'
@@ -124,7 +124,7 @@ if      (defined($dirty)) {
 } elsif (defined($hash1)) {
     print F "#define CANU_VERSION           \"Canu snapshot ($hash1)\\n\"\n";
 } else {
-    print F "#define CANU_VERSION           \"Canu $major.$minor\\n\"\n";
+    print F "#define CANU_VERSION           \"Canu $label v$major.$minor\\n\"\n";
 }
 
 close(F);
