@@ -59,7 +59,6 @@ require Exporter;
              skipStage
              emitStage
              touch
-             makeExecutable
              getInstallDirectory
              getJobIDShellCode
              getLimitShellCode
@@ -82,7 +81,6 @@ use Cwd qw(getcwd);
 use Carp qw(longmess);
 
 use POSIX ":sys_wait_h";  #  For waitpid(..., &WNOHANG)
-use File::Basename;
 use List::Util qw(min max);
 use File::Path 2.08 qw(make_path remove_tree);
 use File::Spec;
@@ -169,9 +167,8 @@ sub schedulerRun () {
     }
 }
 
-sub schedulerFinish ($$) {
+sub schedulerFinish ($) {
     my $dir = shift @_;
-    my $nam = shift @_;
     my $child;
     my @newProcesses;
     my $remain;
@@ -182,8 +179,8 @@ sub schedulerFinish ($$) {
     my $diskfree  = (defined($dir)) ? (diskSpace($dir)) : (0);
 
     print STDERR "----------------------------------------\n";
-    print STDERR "-- Starting '$nam' concurrent execution on ", scalar(localtime()), " with $diskfree GB free disk space ($remain processes; $numberOfProcesses concurrently)\n"  if  (defined($dir));
-    print STDERR "-- Starting '$nam' concurrent execution on ", scalar(localtime()), " ($remain processes; $numberOfProcesses concurrently)\n"                                    if (!defined($dir));
+    print STDERR "-- Starting concurrent execution on ", scalar(localtime()), " with $diskfree GB free disk space ($remain processes; $numberOfProcesses concurrently)\n"  if  (defined($dir));
+    print STDERR "-- Starting concurrent execution on ", scalar(localtime()), " ($remain processes; $numberOfProcesses concurrently)\n"                                    if (!defined($dir));
     print STDERR "\n";
     print STDERR "    cd $dir\n";
 
@@ -245,13 +242,6 @@ sub touch ($@) {
 
 
 
-sub makeExecutable ($) {
-    my $file = shift @_;
-
-    chmod(0755 & ~umask(), $file);
-}
-
-
 #
 #  State management
 #
@@ -278,7 +268,7 @@ sub emitStage ($$@) {
     generateReport($asm);
 
     if (!defined($attempt)) {
-        #print STDERR "-- Finished stage '$stage', reset canuIteration.\n";
+        print STDERR "-- Finished stage '$stage', reset canuIteration.\n";
         setGlobal("canuIteration", 0);
     }
 }
@@ -356,49 +346,72 @@ sub getJobIDShellCode () {
 #  Emits a block of shell code to change shell imposed limit on the number of open files and
 #  processes.
 #
-sub getLimitShellCode () {
+sub getLimitShellCode ($) {
+    my $which = shift @_;
     my $string;
 
-    $string .= "echo \"\"\n";
-    $string .= "echo \"Attempting to increase maximum allowed processes and open files.\"";
-    $string .= "\n";
-    $string .= "max=`ulimit -Hu`\n";
-    $string .= "bef=`ulimit -Su`\n";
-    $string .= "if [ \$bef -lt \$max ] ; then\n";
-    $string .= "  ulimit -Su \$max\n";
-    $string .= "  aft=`ulimit -Su`\n";
-    $string .= "  echo \"  Changed max processes per user from \$bef to \$aft (max \$max).\"\n";
-    $string .= "else\n";
-    $string .= "  echo \"  Max processes per user limited to \$bef, no increase possible.\"\n";
-    $string .= "fi\n";
-    $string .= "\n";
-    $string .= "max=`ulimit -Hn`\n";
-    $string .= "bef=`ulimit -Sn`\n";
-    $string .= "if [ \$bef -lt \$max ] ; then\n";
-    $string .= "  ulimit -Sn \$max\n";
-    $string .= "  aft=`ulimit -Sn`\n";
-    $string .= "  echo \"  Changed max open files from \$bef to \$aft (max \$max).\"\n";
-    $string .= "else\n";
-    $string .= "  echo \"  Max open files limited to \$bef, no increase possible.\"\n";
-    $string .= "fi\n";
-    $string .= "\n";
-    $string .= "echo \"\"\n";
-    $string .= "\n";
+    if ($which eq "processes") {
+        $string .= "\n";
+        $string .= "max=`ulimit -Hu`\n";
+        $string .= "bef=`ulimit -Su`\n";
+        $string .= "if [ \$bef -lt \$max ] ; then\n";
+        $string .= "  ulimit -Su \$max\n";
+        $string .= "  aft=`ulimit -Su`\n";
+        $string .= "  echo \"Changed max processes per user from \$bef to \$aft (max \$max).\"\n";
+        $string .= "  echo \"\"\n";
+        $string .= "else\n";
+        $string .= "  echo \"Max processes per user limited to \$bef, no increase possible.\"\n";
+        $string .= "  echo \"\"\n";
+        $string .= "fi\n";
+        $string .= "\n";
+    }
+
+    if ($which eq "files") {
+        $string .= "\n";
+        $string .= "max=`ulimit -Hn`\n";
+        $string .= "bef=`ulimit -Sn`\n";
+        $string .= "if [ \$bef -lt \$max ] ; then\n";
+        $string .= "  ulimit -Sn \$max\n";
+        $string .= "  aft=`ulimit -Sn`\n";
+        $string .= "  echo \"Changed max open files from \$bef to \$aft (max \$max).\"\n";
+        $string .= "  echo \"\"\n";
+        $string .= "else\n";
+        $string .= "  echo \"Max open files limited to \$bef, no increase possible.\"\n";
+        $string .= "  echo \"\"\n";
+        $string .= "fi\n";
+        $string .= "\n";
+    }
 
     return($string);
 }
 
 
-#  Used inside canu to find where binaries are located.
+#  Used inside canu to find where binaries are located.  It uses uname to find OS, architecture and
+#  system name, then uses that to construct a path to binaries.  If a "pathMap" is defined, this is
+#  used to hardcode a path to a system name.
 #
 sub getBinDirectory () {
     my $installDir = getInstallDirectory();
 
-    my $syst = `uname -s`;                          chomp $syst;  #  OS implementation
-    my $arch = `uname -m | sed s/x86_64/amd64/`;    chomp $arch;  #  Hardware platform
+    my $syst = `uname -s`;    chomp $syst;  #  OS implementation
+    my $arch = `uname -m`;    chomp $arch;  #  Hardware platform
+    my $name = `uname -n`;    chomp $name;  #  Name of the system
+
+    $arch = "amd64"  if ($arch eq "x86_64");
+    $arch = "ppc"    if ($arch eq "Power Macintosh");
 
     my $path = "$installDir/$syst-$arch/bin";
 
+    my $pathMap = getGlobal("pathMap");
+    if (defined($pathMap)) {
+        open(F, "< $pathMap") or caFailure("failed to open pathMap '$pathMap'", undef);
+        while (<F>) {
+            my ($n, $b) = split '\s+', $_;
+            $path = $b if ($name eq $n);
+        }
+        close(F);
+    }
+
     if (! -d "$path") {
         $path = $installDir;
     }
@@ -414,25 +427,33 @@ sub getBinDirectoryShellCode () {
     my $installDir = getInstallDirectory();
     my $string;
 
-    #  First, run any preExec command that might exist.
-
-    if (defined(getGlobal("preExec"))) {
-        $string .= "#  Pre-execution commands.\n";
-        $string .= "\n";
-        $string .= getGlobal('preExec') . "\n";
-        $string .= "\n";
-    }
-
-    #  Then, setup paths.
-
-    $string .= "\n";
-    $string .= "#  Path to Canu.\n";
-    $string .= "\n";
     $string .= "syst=`uname -s`\n";
-    $string .= "arch=`uname -m | sed s/x86_64/amd64/`\n";
+    $string .= "arch=`uname -m`\n";
+    $string .= "name=`uname -n`\n";
+    $string .= "\n";
+    $string .= "if [ \"\$arch\" = \"x86_64\" ] ; then\n";
+    $string .= "  arch=\"amd64\"\n";
+    $string .= "fi\n";
+    $string .= "if [ \"\$arch\" = \"Power Macintosh\" ] ; then\n";
+    $string .= "  arch=\"ppc\"\n";
+    $string .= "fi\n";
     $string .= "\n";
     $string .= "bin=\"$installDir/\$syst-\$arch/bin\"\n";
     $string .= "\n";
+
+    my $pathMap = getGlobal("pathMap");
+    if (defined($pathMap)) {
+        open(PM, "< $pathMap") or caFailure("failed to open pathMap '$pathMap'", undef);
+        while (<PM>) {
+            my ($n, $b) = split '\s+', $_;
+            $string .= "if [ \"\$name\" = \"$n\" ] ; then\n";
+            $string .= "  bin=\"$b\"\n";
+            $string .= "fi\n";
+        }
+        close(PM);
+        $string .= "\n";
+    }
+
     $string .= "if [ ! -d \"\$bin\" ] ; then\n";
     $string .= "  bin=\"$installDir\"\n";
     $string .= "fi\n";
@@ -617,17 +638,20 @@ sub submitScript ($$) {
 
     open(F, "> $script") or caFailure("failed to open '$script' for writing", undef);
     print F "#!" . getGlobal("shell") . "\n";
-    print F "\n";
+    print F "\n"                                                                        if (getGlobal("gridEngine") eq "SGE");
     print F "#  Attempt to (re)configure SGE.  For unknown reasons, jobs submitted\n"   if (getGlobal("gridEngine") eq "SGE");
     print F "#  to SGE, and running under SGE, fail to read the shell init scripts,\n"  if (getGlobal("gridEngine") eq "SGE");
     print F "#  and so they don't set up SGE (or ANY other paths, etc) properly.\n"     if (getGlobal("gridEngine") eq "SGE");
     print F "#  For the record, interactive logins (qlogin) DO set the environment.\n"  if (getGlobal("gridEngine") eq "SGE");
-    print F "\n"                                                                        if (getGlobal("gridEngine") eq "SGE");
-    print F "if [ \"x\$SGE_ROOT\" != \"x\" -a \\\n"                                     if (getGlobal("gridEngine") eq "SGE");
-    print F "     -e  \$SGE_ROOT/\$SGE_CELL/common/settings.sh ]; then\n"               if (getGlobal("gridEngine") eq "SGE");
+    print F "\n";
+    print F "if [ \"x\$SGE_ROOT\" != \"x\" ]; then \n"                                  if (getGlobal("gridEngine") eq "SGE");
     print F "  . \$SGE_ROOT/\$SGE_CELL/common/settings.sh\n"                            if (getGlobal("gridEngine") eq "SGE");
     print F "fi\n"                                                                      if (getGlobal("gridEngine") eq "SGE");
     print F "\n";
+    print F "#  On the off chance that there is a pathMap, and the host we\n";
+    print F "#  eventually get scheduled on doesn't see other hosts, we decide\n";
+    print F "#  at run time where the binary is.\n";
+    print F "\n";
     print F getBinDirectoryShellCode();
     print F "\n";
     print F setWorkDirectoryShellCode(".");
@@ -636,10 +660,10 @@ sub submitScript ($$) {
     print F "ln -s canu-scripts/canu.$idx.out canu.out\n";
     print F "\n";
     print F "/usr/bin/env perl \\\n";
-    print F "\$bin/" . basename($0) . " " . getCommandLineOptions() . " canuIteration=" . getGlobal("canuIteration") . "\n";
+    print F "\$bin/canu " . getCommandLineOptions() . " canuIteration=" . getGlobal("canuIteration") . "\n";
     close(F);
 
-    makeExecutable("$script");
+    system("chmod +x $script");
 
     #  Construct a submission command line.
 
@@ -664,22 +688,13 @@ sub submitScript ($$) {
 
     $gridOpts  = $jobHold;
     $gridOpts .= " "                                  if (defined($gridOpts));
-    # LSF takes the first argument to specify them in reverse order of preference, otherwise append
-    if (uc(getGlobal("gridEngine")) eq "LSF") {
-       $gridOpts .= getGlobal("gridOptionsExecutive")    if (defined(getGlobal("gridOptionsExecutive")));
-       $gridOpts .= " "                                  if (defined($gridOpts));
-       $gridOpts .= getGlobal("gridOptions")             if (defined(getGlobal("gridOptions")));
-       $gridOpts .= " "                                  if (defined($gridOpts));
-    }
     $gridOpts .= $memOption                           if (defined($memOption));
     $gridOpts .= " "                                  if (defined($gridOpts));
     $gridOpts .= $thrOption                           if (defined($thrOption));
-    if (uc(getGlobal("gridEngine")) ne "LSF") {
-       $gridOpts .= " "                                  if (defined($gridOpts));
-       $gridOpts .= getGlobal("gridOptions")             if (defined(getGlobal("gridOptions")));
-       $gridOpts .= " "                                  if (defined($gridOpts));
-       $gridOpts .= getGlobal("gridOptionsExecutive")    if (defined(getGlobal("gridOptionsExecutive")));
-    }
+    $gridOpts .= " "                                  if (defined($gridOpts));
+    $gridOpts .= getGlobal("gridOptions")             if (defined(getGlobal("gridOptions")));
+    $gridOpts .= " "                                  if (defined($gridOpts));
+    $gridOpts .= getGlobal("gridOptionsExecutive")    if (defined(getGlobal("gridOptionsExecutive")));
 
     my $submitCommand        = getGlobal("gridEngineSubmitCommand");
     my $nameOption           = getGlobal("gridEngineNameOption");
@@ -723,6 +738,7 @@ sub buildGridArray ($$$$) {
 
     if (uc(getGlobal("gridEngine")) eq "PBS") {
         $off = "-F \"$off\"";
+        $off = "";
     }
 
     $opt =~ s/ARRAY_NAME/$name/g;        #  Replace ARRAY_NAME with 'job name'
@@ -828,18 +844,6 @@ sub buildThreadOption ($) {
 }
 
 
-sub purgeGridJobSubmitScripts ($$) {
-    my $path    = shift @_;
-    my $script  = shift @_;
-    my $idx     = "01";
-
-    while (-e "$path/$script.jobSubmit-$idx.sh") {
-        unlink "$path/$script.jobSubmit-$idx.sh";
-        $idx++;
-    }
-}
-
-
 sub buildGridJob ($$$$$$$$$) {
     my $asm     = shift @_;
     my $jobType = shift @_;
@@ -890,18 +894,12 @@ sub buildGridJob ($$$$$$$$$) {
     $opts .= "$outputOption "   if (defined($outputOption));
     $opts =~ s/\s+$//;
 
-    #  Find a unique file name to save the command.
-
-    my $idx = "01";
-
-    while (-e "$path/$script.jobSubmit-$idx.sh") {
-        $idx++;
-    }
-
     #  Build and save the command line.  Return the command PREFIX (we'll be adding .sh and .out as
     #  appropriate), and the job name it will be submitted with (which isn't expected to be used).
 
-    open(F, "> $path/$script.jobSubmit-$idx.sh") or die;
+    my $cmd;
+
+    open(F, "> $path/$script.jobSubmit.sh") or die;
     print F "#!/bin/sh\n";
     print F "\n";
     print F "$submitCommand \\\n";
@@ -909,12 +907,12 @@ sub buildGridJob ($$$$$$$$$) {
     print F "  $nameOption \"$jobName\" \\\n";
     print F "  $arrayOpt \\\n";
     print F "  ./$script.sh $arrayOff \\\n";
-    print F "> ./$script.jobSubmit-$idx.out 2>&1\n";
+    print F "> ./$script.jobSubmit.out 2>&1\n";
     close(F);
 
-    makeExecutable("$path/$script.jobSubmit-$idx.sh");
+    chmod 0755, "$path/$script.jobSubmit.sh";
 
-    return("$script.jobSubmit-$idx", $jobName);
+    return("$script.jobSubmit", $jobName);
 }
 
 
@@ -1031,7 +1029,7 @@ sub submitOrRunParallelJob ($$$$@) {
 
     #  The script MUST be executable.
 
-    makeExecutable("$path/$script.sh");
+    system("chmod +x \"$path/$script.sh\"");
 
     #  Report what we're doing.
 
@@ -1093,8 +1091,6 @@ sub submitOrRunParallelJob ($$$$@) {
 
         print STDERR "--\n";
 
-        purgeGridJobSubmitScripts($path, $script);
-
         foreach my $j (@jobs) {
             my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
 
@@ -1205,8 +1201,6 @@ sub submitOrRunParallelJob ($$$$@) {
         print STDERR "Please run the following commands to submit jobs to the grid for execution using $mem gigabytes memory and $thr threads:\n";
         print STDERR "\n";
 
-        purgeGridJobSubmitScripts($path, $script);
-
         foreach my $j (@jobs) {
             my  $cwd = getcwd();
             my ($cmd, $jobName) = buildGridJob($asm, $jobType, $path, $script, $mem, $thr, $dsk, $j, undef);
@@ -1253,7 +1247,7 @@ sub submitOrRunParallelJob ($$$$@) {
     my $nParallel  = $nCParallel < $nMParallel ? $nCParallel : $nMParallel;
 
     schedulerSetNumberOfProcesses($nParallel);
-    schedulerFinish($path, $jobType);
+    schedulerFinish($path);
 }
 
 
@@ -1298,7 +1292,6 @@ sub reportRunError ($) {
 
     #  The rest is rather straightforward at least.
 
-    print STDERR "\n";
     print STDERR "ERROR:\n";
 
     if      ($rc ==  -1) {
@@ -1437,40 +1430,34 @@ sub findExecutable ($) {
 
 #  Use caExit() for transient errors, like not opening files, processes that die, etc.
 sub caExit ($$) {
-    my  $asm     = getGlobal("onExitNam");
-    my  $msg     = shift @_;
-    my  $log     = shift @_;
-    my  $version = getGlobal("version");
+    my  $asm   = getGlobal("onExitNam");
+    my  $msg   = shift @_;
+    my  $log   = shift @_;
 
+    print STDERR "================================================================================\n";
+    print STDERR "Don't panic, but a mostly harmless error occurred and Canu stopped.\n";
     print STDERR "\n";
-    print STDERR "ABORT:\n";
-    print STDERR "ABORT: $version\n";
-    print STDERR "ABORT: Don't panic, but a mostly harmless error occurred and Canu stopped.\n";
-    print STDERR "ABORT: Try restarting.  If that doesn't work, ask for help.\n";
-    print STDERR "ABORT:\n";
-    print STDERR "ABORT:   $msg.\n"     if (defined($msg));
-    print STDERR "ABORT:\n"             if (defined($msg));
 
-    if (defined($log) && -e $log) {
+    if (defined($log)) {
         my  $df = diskSpace($log);
 
-        print STDERR "ABORT: Disk space available:  $df GB\n";
-        print STDERR "ABORT:\n";
+        print STDERR "Disk space available:  $df GB\n";
+        print STDERR "\n";
     }
 
     if (-e $log) {
-        print STDERR "ABORT: Last 50 lines of the relevant log file ($log):\n";
-        print STDERR "ABORT:\n";
-
-        open(Z, "tail -n 50 $log");
-        while (<Z>) {
-            print STDERR "ABORT: $_";
-        }
-        close(Z);
-
-        print STDERR "ABORT:\n";
+        print STDERR "Last 50 lines of the relevant log file ($log):\n";
+        print STDERR "\n";
+        system("tail -n 50 $log");
+        print STDERR "\n";
     }
 
+    my $version = getGlobal("version");
+
+    print STDERR "$version failed with:\n";
+    print STDERR "  $msg\n";
+    print STDERR "\n";
+
     my $fail = getGlobal('onFailure');
     if (defined($fail)) {
         runCommandSilently(getGlobal("onExitDir"), "$fail $asm", 0);
@@ -1486,36 +1473,27 @@ sub caFailure ($$) {
     my  $msg     = shift @_;
     my  $log     = shift @_;
     my  $version = getGlobal("version");
-    my  $trace   = longmess("Failed");
-
-    $trace =~ s/\n/\nCRASH: /g;
+    my  $trace   = longmess(undef);
 
+    print STDERR "================================================================================\n";
+    print STDERR "Please panic.  Canu failed, and it shouldn't have.\n";
+    print STDERR "\n";
+    print STDERR "Stack trace:\n";
+    print STDERR "\n";
+    print STDERR "$trace\n";
     print STDERR "\n";
-    print STDERR "CRASH:\n";
-    print STDERR "CRASH: $version\n";
-    print STDERR "CRASH: Please panic, this is abnormal.\n";
-    print STDERR "ABORT:\n";
-    print STDERR "CRASH:   $msg.\n";
-    print STDERR "CRASH:\n";
-    print STDERR "CRASH: $trace\n";
-    #print STDERR "CRASH:\n";   #  $trace has an extra CRASH: at the end
 
     if (-e $log) {
-        print STDERR "CRASH: Last 50 lines of the relevant log file ($log):\n";
-        print STDERR "CRASH:\n";
-
-        open(Z, "tail -n 50 $log");
-        while (<Z>) {
-            print STDERR "CRASH: $_";
-        }
-        close(Z);
-
-        print STDERR "CRASH:\n";
-    } else {
-        print STDERR "CRASH: No log file supplied.\n";
-        print STDERR "CRASH:\n";
+        print STDERR "Last few lines of the relevant log file ($log):\n";
+        print STDERR "\n";
+        system("tail -n 50 $log");
     }
 
+    print STDERR "\n";
+    print STDERR "$version failed with:\n";
+    print STDERR "  $msg\n";
+    print STDERR "\n";
+
     my $fail = getGlobal('onFailure');
     if (defined($fail)) {
         runCommandSilently(getGlobal("onExitDir"), "$fail $asm", 0);
