@@ -40,7 +40,7 @@ package canu::Meryl;
 require Exporter;
 
 @ISA    = qw(Exporter);
-@EXPORT = qw(merylConfigure merylCheck merylProcess merylSubtract merylFinishSubtraction);
+@EXPORT = qw(merylConfigure merylCheck merylProcess);
 
 use strict;
 
@@ -52,9 +52,9 @@ use canu::Defaults;
 use canu::Execution;
 use canu::Gatekeeper;
 use canu::ErrorEstimate;
+use canu::HTML;
 use canu::Report;
 use canu::Grid_Cloud;
-use canu::HaplotypeReads qw(getHaplotypes);
 
 
 
@@ -290,7 +290,7 @@ sub merylPlotHistogram ($$$$) {
 
     if (runCommandSilently($path, "$gnuplot ./$ofile.histogram.$suffix.gp > /dev/null 2>&1", 0)) {
         print STDERR "--\n";
-        print STDERR "-- WARNING: gnuplot failed.\n";
+        print STDERR "-- WARNING: gnuplot failed; no plots will appear in HTML output.\n";
         print STDERR "--\n";
         print STDERR "----------------------------------------\n";
     }
@@ -309,13 +309,11 @@ sub merylParameters ($$) {
 
     #  Find a place to run stuff.
 
-    $base = "haplotype"   if ($tag eq "hap");
     $base = "correction"  if ($tag eq "cor");
     $base = "trimming"    if ($tag eq "obt");
     $base = "unitigging"  if ($tag eq "utg");
 
     $path = "$base/0-mercounts";
-    $path = "$path-$asm" if ($tag eq "hap");
 
     #  Decide on which set of parameters we need to be using, and make output file names.
 
@@ -415,7 +413,7 @@ sub merylConfigure ($$) {
 
     my $mem = int(getGlobal("merylMemory")  * 1024 * 0.8);   #  Because meryl expects megabytes, not gigabytes.
     my $thr = getGlobal("merylThreads");
-    my $cov = getExpectedCoverage($tag, $asm);
+    my $cov = getExpectedCoverage($base, $asm);
 
     caExit("merylMemory isn't defined?", undef)   if (!defined($mem));
     caExit("merylThreads isn't defined?", undef)  if (!defined($thr));
@@ -480,11 +478,11 @@ sub merylConfigure ($$) {
 
     close(F);
 
-    makeExecutable("$path/meryl.sh");
     stashFile("$path/meryl.sh");
 
   finishStage:
     emitStage($asm, "merylConfigure");
+    buildHTML($asm, $tag);
 
   allDone:
 }
@@ -516,24 +514,24 @@ sub merylCheck ($$) {
     #  shows how to process multiple jobs.  This only checks for the existence of the final outputs.
     #  (unitigger is the same)
 
-    #  If too many attempts, give up.
+    #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-    if ($attempt >= getGlobal("canuIterationMax")) {
+    if ($attempt > 1) {
         print STDERR "--\n";
-        print STDERR "-- Meryl failed, tried $attempt times, giving up.\n";
+        print STDERR "-- meryl failed.\n";
         print STDERR "--\n";
-        caExit(undef, undef);
     }
 
-    if ($attempt > 0) {
-        print STDERR "--\n";
-        print STDERR "-- Meryl failed, retry.\n";
-        print STDERR "--\n";
+    #  If too many attempts, give up.
+
+    if ($attempt > getGlobal("canuIterationMax")) {
+        caExit("failed to generate mer counts.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
     }
 
     #  Otherwise, run some jobs.
 
     emitStage($asm, "merylCheck", $attempt);
+    buildHTML($asm, $tag);
 
     submitOrRunParallelJob($asm, "meryl", $path, "meryl", (1));
     return;
@@ -549,112 +547,12 @@ sub merylCheck ($$) {
     stashFile("$path/meryl.success");
 
     emitStage($asm, "merylCheck");
+    buildHTML($asm, $tag);
 
   allDone:
 }
 
-sub merylSubtract ($$) {
-    my $asm     = shift @_;
-    my $tag     = shift @_;
 
-    my $bin     = getBinDirectory();
-    my $cmd;
-
-    my ($base, $path, $merSize, $merThresh, $merScale, $merDistinct, $merTotal, $ffile, $ofile) = merylParameters($asm, $tag);
-
-    goto allDone   if (skipStage($asm, "$tag-meryl") == 1);
-    goto allDone   if(fileExists("$path/$asm.ms$merSize.only.mcdat"));
-
-    my $otherHaplotypes = "";
-    my $toMerge = 0;
-
-    my @haplotypes = getHaplotypes($base);
-    foreach my $haplotype (@haplotypes) {
-       if ("$base/0-mercounts-$haplotype" ne $path) {
-          $otherHaplotypes .= "-s ../0-mercounts-$haplotype/$haplotype.ms$merSize";
-          $toMerge++;
-       }
-    }
-    if ($toMerge > 1) {
-       # run merge of other haplotypes, to create a single one, update otherHaplotypes to point to that
-       caFailure("Error: more than two haplotypes isn't implemented yet!", "$path");
-    }
-    if (runCommand($path, "$bin/meryl -M difference -s $asm.ms$merSize $otherHaplotypes -o $asm.ms$merSize.only > $asm.difference.out 2> $asm.difference.err")) {
-       caFailure("meryl failed to difference", "$asm.difference.err");
-    }
-    addToReport("${tag}Meryl", merylGenerateHistogram($asm, $tag));
-
-  allDone:
-}
-
-sub merylFinishSubtraction($$) {
-    my $asm     = shift @_;
-    my $tag     = shift @_;
-
-    my $bin     = getBinDirectory();
-    my $cmd;
-
-    my ($base, $path, $merSize, $merThresh, $merScale, $merDistinct, $merTotal, $ffile, $ofile) = merylParameters($asm, $tag);
-
-    goto allDone      if (skipStage($asm, "$tag-meryl") == 1);
-    goto allDone      if (fileExists("$path/$ofile.threshold"));
-    goto finishStage  if (fileExists("$path/$ofile.mcidx") && fileExists("$path/$ofile.mcdat"));
-
-  finishStage:
-
-    # also figure out the histogram info for next step
-    fetchFile("$path/$ofile.histogram");
-
-    my $d = 0;
-    my $prevD = -1;
-    my $prevDPrime = 0;
-    my $dPrime = 0;
-    my $minCount = 0;
-    my $minCov = 0;
-    my $maxCov = 0;
-    my $prevCount = 0;
-
-    open(F, "< $path/$ofile.histogram") or caFailure("failed to read mer histogram from '$path/$ofile.histogram'", undef);
-    while (<F>) {
-       my ($threshold, $num, $distinct, $total) = split '\s+', $_;
-       if ($prevD == -1) {
-          $prevD = $num;
-          $prevCount = $num;
-        } else {
-           $d = $num - $prevCount;
-           $dPrime = $d - $prevD;
-           if ($d * $prevD < 0) {
-              if ($d > $prevD) {
-                 $minCov = $threshold-1 if ($minCov == 0);
-                 $minCount = $prevCount;
-              }
-           }
-           if ($threshold - 5 > $minCov && $num < $minCount*0.75) {
-              $maxCov = $threshold;
-              last;
-           }
-           $prevCount = $num;
-           $prevD = $d;
-           $prevDPrime = $dPrime;
-        }
-    }
-    close(F);
-
-    print STDERR "-- Meryl finished successfully with threshold $minCov to $maxCov for $asm.\n";
-    open(F, "> $path/$ofile.threshold") or caExit("can't open '$path/$ofile.threshold' for writing: $!", undef);
-    printf (F "$minCov\t$maxCov\n");
-    close(F);
-
-    stashFile("$path/$ofile.threshold");
-
-    unlink "$path/$ofile.mcidx"   if (getGlobal("saveMerCounts") == 0);
-    unlink "$path/$ofile.mcdat"   if (getGlobal("saveMerCounts") == 0);
-
-    emitStage($asm, "$tag-meryl");
-
-  allDone:
-     stopAfter("meryl");
-}
 
 sub merylProcess ($$) {
     my $asm     = shift @_;
@@ -770,38 +668,27 @@ sub merylProcess ($$) {
         caFailure("didn't find any mers?", "$path/$ofile.histogram.info")  if ($totalMers == 0);
 
         my $filterThreshold = getGlobal("${tag}MhapFilterThreshold");
-        my $misRate         = 0.05;
-        my $minCovThresh    = 5.00; # should be a parameter
-        my $cov             = int($minCovThresh*getExpectedCoverage($tag, $asm));
-        my $minCount        = ($cov > int($filterThreshold * $totalMers) ? $cov : int($filterThreshold * $totalMers));
-        my $repeatThreshold = $minCount;
+        my $misRate         = 0.1;
+        my $minCount        = int($filterThreshold * $totalMers);
         my $totalToOutput   = 0;
-        my $totalToFilter   = 0;
         my $totalFiltered   = 0;
 
         if (defined(getGlobal("${tag}MhapFilterUnique"))) {
-            $minCount = uniqueKmerThreshold($base, $asm, $tag, $merSize, $misRate) + 1;
-        }
-        # if the threshold was too low, update it
-        if ($cov > int($filterThreshold * $totalMers)) {
-           printf STDERR "-- For %s overlapping, the threshold %0.15f is too low, resetting to %0.15f to capture %f * %d coverage.\n", getGlobal("${tag}Overlapper"), getGlobal("${tag}MhapFilterThreshold"), ($cov / $totalMers), $minCovThresh, getExpectedCoverage($tag, $asm);
-           setGlobal("${tag}MhapFilterThreshold", $cov / $totalMers);
+            $minCount = uniqueKmerThreshold($base, $asm, $merSize, $misRate) + 1;
         }
 
         open(F, "< $path/$ofile.histogram") or die "Failed to open '$path/$ofile.histogram' for reading: $!\n";
         while (<F>) {
            my ($kCount, $occurences, $cumsum, $faction) = split '\s+', $_;
-           if ($kCount < $minCount && defined(getGlobal("${tag}MhapFilterUnique"))) {
+           if ($kCount < $minCount) {
               $totalFiltered = $cumsum * 100;
-              $totalToFilter += $occurences;
            }
-           if ($kCount >= $repeatThreshold) {
+           if ($kCount >= $minCount) {
               $totalToOutput += $occurences;
            }
         }
         close(F);
         $totalToOutput *= 2; # for the reverse complement
-        $totalToFilter *= 2; # for the reverse complement
 
         fetchFile("$path/$ofile.mcdat");
         fetchFile("$path/$ofile.mcidx");
@@ -809,7 +696,7 @@ sub merylProcess ($$) {
         open(F, "$bin/meryl -Dt -n $minCount -s $path/$ofile | ")    or die "Failed to run meryl to generate frequent mers $!\n";
         open(O, "| gzip -c > $path/$ofile.frequentMers.ignore.gz")   or die "Failed to open '$path/$ofile.frequentMers.ignore.gz' for writing: $!\n";
 
-        printf(O "%d\t%d\n", $totalToFilter, $totalToOutput);
+        printf(O "%d\n", $totalToOutput);
 
         while (!eof(F)) {
             my $h = <F>;
@@ -829,9 +716,9 @@ sub merylProcess ($$) {
         stashFile("$path/$ffile");
 
         if (defined(getGlobal("${tag}MhapFilterUnique"))) {
-           printf STDERR "-- For %s overlapping, filtering low-occurence k-mers < %d (%.2f\%) based on estimated error of %.2f\%.\n", getGlobal("${tag}Overlapper"), $minCount, $totalFiltered, 100*estimateError($asm, $tag, $merSize);
+           printf STDERR "-- For %s overlapping, filtering low-occurence k-mers < %d (%.2f\%) based on estimated error of %.2f\%.\n", getGlobal("${tag}Overlapper"), $minCount, $totalFiltered, 100*estimateRawError($base, $asm, $tag, $merSize);
         }
-        printf STDERR "-- For %s overlapping, set repeat k-mer threshold to %d.\n", getGlobal("${tag}Overlapper"),  ($cov > int($filterThreshold * $totalMers) ? $cov : int($filterThreshold * $totalMers));
+        printf STDERR "-- For %s overlapping, set repeat k-mer threshold to %d.\n", getGlobal("${tag}Overlapper"), int($filterThreshold * $totalMers);
     }
 
     #  Report the new threshold.
@@ -876,6 +763,7 @@ sub merylProcess ($$) {
     unlink "$path/$ofile.mcdat"   if (getGlobal("saveMerCounts") == 0);
 
     emitStage($asm, "$tag-meryl");
+    buildHTML($asm, $tag);
 
   allDone:
     stopAfter("meryl");
