@@ -40,7 +40,7 @@ package canu::OverlapBasedTrimming;
 require Exporter;
 
 @ISA    = qw(Exporter);
-@EXPORT = qw(qualTrimReads dedupeReads trimReads splitReads loadTrimmedReads dumpTrimmedReads);
+@EXPORT = qw(qualTrimReads dedupeReads trimReads splitReads dumpReads);
 
 use strict;
 
@@ -50,6 +50,7 @@ use canu::Defaults;
 use canu::Execution;
 use canu::Gatekeeper;
 use canu::Report;
+use canu::HTML;
 use canu::Grid_Cloud;
 
 
@@ -117,6 +118,7 @@ sub trimReads ($) {
 
   finishStage:
     emitStage($asm, "obt-trimReads");
+    buildHTML($asm, "obt");
 
   allDone:
 }
@@ -187,13 +189,14 @@ sub splitReads ($) {
 
   finishStage:
     emitStage($asm, "obt-splitReads");
+    buildHTML($asm, "obt");
 
   allDone:
 }
 
 
 
-sub loadTrimmedReads ($) {
+sub dumpReads ($) {
     my $asm    = shift @_;
     my $bin    = getBinDirectory();
     my $cmd;
@@ -201,84 +204,42 @@ sub loadTrimmedReads ($) {
     my $inp;
 
     goto allDone   if (skipStage($asm, "obt-dumpReads") == 1);
-    goto allDone   if (getNumberOfBasesInStore("utg", $asm) > 0);
+    goto allDone   if (sequenceFileExists("$asm.trimmedReads"));
 
     make_path($path)  if (! -d $path);
 
     fetchFile("./trimming/3-overlapbasedtrimming/$asm.1.trimReads.clear");
     fetchFile("./trimming/3-overlapbasedtrimming/$asm.2.splitReads.clear");
 
-    $inp = "./$asm.1.trimReads.clear"   if (-e "$path/$asm.1.trimReads.clear");
-    $inp = "./$asm.2.splitReads.clear"  if (-e "$path/$asm.2.splitReads.clear");
+    $inp = "./3-overlapbasedtrimming/$asm.1.trimReads.clear"   if (-e "$path/$asm.1.trimReads.clear");
+    $inp = "./3-overlapbasedtrimming/$asm.2.splitReads.clear"  if (-e "$path/$asm.2.splitReads.clear");
 
-    caFailure("loading trimmed reads failed; no 'clear' input", "trimming/$asm.trimmedReads.err")  if (!defined($inp));
+    caFailure("dumping trimmed reads failed; no 'clear' input", "trimming/$asm.trimmedReads.err")  if (!defined($inp));
 
-    $cmd  = "$bin/loadTrimmedReads \\\n";
-    $cmd .= "  -G ../$asm.gkpStore \\\n";
+    $cmd  = "$bin/gatekeeperDumpFASTQ -fasta -nolibname \\\n";
+    $cmd .= "  -G ./$asm.gkpStore \\\n";
     $cmd .= "  -c $inp \\\n";
-    $cmd .= "> ./$asm.loadtrimmedReads.err 2>&1";
+    $cmd .= "  -o ../$asm.trimmedReads.gz \\\n";     #  Adds .fasta
+    $cmd .= ">    ../$asm.trimmedReads.err 2>&1";
 
-    if (runCommand($path, $cmd)) {
-        caFailure("loading clear ranges failed", "./$asm.trimmedReads.err");
+    if (runCommand("trimming", $cmd)) {
+        caFailure("dumping trimmed reads failed", "./$asm.trimmedReads.err");
     }
 
     unlink("./$asm.trimmedReads.err");
 
-    #  Report reads.
+    stashFile("./$asm.trimmedReads.fasta.gz");
 
-    addToReport("utgGkpStore", generateReadLengthHistogram("utg", $asm));
-
-    #stashFile("./$asm.trimmedReads.fasta.gz");
-
-    if (getGlobal("saveOverlaps") eq "1") {
-        print STDERR "--\n";
-        print STDERR "-- Purging overlaps used for trimming.\n";
-
-        remove_tree("trimming/$asm.ovlStore")
-    } else {
-        print STDERR "--\n";
-        print STDERR "-- Overlaps used for trimming saved.\n";
-    }
+    #  Need gatekeeperDumpFASTQ to also write a gkp input file
+    #touch("../$asm.trimmedReads.gkp");
 
   finishStage:
     emitStage($asm, "obt-dumpReads");
+    buildHTML($asm, "obt");
 
   allDone:
-    stopAfter("readTrimming");
-}
-
-
-
-sub dumpTrimmedReads ($) {
-    my $asm     = shift @_;
-    my $bin     = getBinDirectory();
-    my $cmd;
-
-    goto allDone   if (skipStage($asm, "obt-dumpTrimmedReads") == 1);
-    goto allDone   if (sequenceFileExists("$asm.trimmedReads"));
-    goto allDone   if (getGlobal("saveReads") == 0);
-    return         if (! -d "trimming/$asm.gkpStore");  #  No trimming done, nothing to do.
-
-    $cmd  = "$bin/gatekeeperDumpFASTQ \\\n";
-    $cmd .= "  -trimmed \\\n";
-    $cmd .= "  -G ./$asm.gkpStore \\\n";
-    $cmd .= "  -o ./$asm.trimmedReads.gz \\\n";
-    $cmd .= "  -fasta \\\n";
-    $cmd .= "  -nolibname \\\n";
-    $cmd .= "> ./$asm.trimmedReads.fasta.err 2>&1";
-
-    if (runCommand(".", $cmd)) {
-        caExit("failed to output trimmed reads", "./$asm.trimmedReads.fasta.err");
-    }
-
-    unlink "./$asm.trimmedReads.fasta.err";
-
     print STDERR "--\n";
-    print STDERR "-- Trimmed reads saved in '", sequenceFileExists("$asm.trimmedReads"), "'.\n";
+    print STDERR "-- Trimmed reads saved in 'trimming/$asm.trimmedReads.fasta.gz'\n";
 
-  finishStage:
-    emitStage($asm, "cor-dumpTrimmedReads");
-
-  allDone:
     stopAfter("readTrimming");
 }
