@@ -50,73 +50,12 @@ use canu::Defaults;
 use canu::Execution;
 use canu::Gatekeeper;
 use canu::Report;
+use canu::HTML;
 use canu::Grid_Cloud;
 
 #  Hardcoded to use utgOvlErrorRate
 
 
-sub loadReadLengthsAndNumberOfOverlaps ($$$$) {
-    my $asm     = shift @_;
-    my $maxID   = shift @_;
-
-    my $rlVec   = shift @_;
-    my $rlCnt   = 0;
-    my $rlSum   = 0;
-
-    my $noVec   = shift @_;
-    my $noCnt   = 0;
-    my $noSum   = 0;
-
-    my $bin     = getBinDirectory();
-
-    $$rlVec     = "\xff" x ($maxID * 4 + 4);
-    $$noVec     = "\xff" x ($maxID * 4 + 4);
-
-    print STDERR "--\n";
-    print STDERR "-- Loading read lengths.\n";
-
-    open(F, "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads |");
-    while (<F>) {
-        s/^\s+//;
-        s/\s+$//;
- 
-        next if (m/^readID/);   #  Header line
-        next if (m/^------/);   #  ------ ----
-
-        my @v = split '\s+', $_;
-
-        vec($$rlVec, $v[0], 32) = $v[2];
-        $rlCnt                 += 1;
-        $rlSum                 += $v[2];
-    }
-    close(F);
-
-    caExit("Failed to load read lengths from '$asm.gkpStore'", undef)   if ($rlCnt == 0);
-
-
-    fetchStore("unitigging/$asm.ovlStore");
-
-    print STDERR "-- Loading number of overlaps per read.\n";
-
-    open(F, "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts |");
-    while (<F>) {
-        s/^\s+//;
-        s/\s+$//;
-
-        my @v = split '\s+', $_;
-
-        vec($$noVec, $v[0], 32) = $v[1];
-        $noCnt                 += 1;
-        $noSum                 += $v[1];
-    }
-    close(F);
-
-    caExit("Failed to load number of overlaps per read from '$asm.ovlStore'", undef)   if ($noCnt == 0);
-
-    return($rlSum, $noSum);
-}
-
-
 
 sub readErrorDetectionConfigure ($) {
     my $asm     = shift @_;
@@ -135,93 +74,78 @@ sub readErrorDetectionConfigure ($) {
 
     make_path("$path")  if (! -d "$path");
 
-    my $maxID = getNumberOfReadsEarliestVersion($asm);
+    #  RED uses 13 bytes/base plus 12 bytes/overlap + space for evidence reads.
 
-    my ($rlVec, $noVec);
-    my ($rlSum, $noSum) = loadReadLengthsAndNumberOfOverlaps($asm, $maxID, \$rlVec, \$noVec);
+    my @readLengths;
+    my @numOlaps;
 
-    if ($noSum == 0) {
-        print STDERR "--\n";
-        print STDERR "-- WARNING:\n";
-        print STDERR "-- WARNING: Found no overlaps.  Disabling Overlap Error Adjustment.\n";
-        print STDERR "-- WARNING:\n";
-        print STDERR "--\n";
-        setGlobal("enableOEA", 0);
-        return;
+    #print STDERR "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads\n";
+    open(F, "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads |");
+    while (<F>) {
+        my @v = split '\s+', $_;
+        $readLengths[$v[0]] = $v[2];
     }
+    close(F);
 
-    #  Find the maximum size of each block of 100,000 reads.  findErrors reads up to 100,000 reads
-    #  to process at one time.  It uses 1 * length + 4 * 100,000 bytes of memory for bases and ID storage,
-    #  and has two buffers of this size.
+    #  NEEDS OPTIMIZE - only need counts here, not the whole store
 
-    my $maxBlockSize = 512 * 1024 * 1024;  #  This is hardcoded in findErrors.C
-    my $maxMem       = getGlobal("redMemory") * 1024 * 1024 * 1024;
-    my $maxReads     = getGlobal("redBatchSize");
-    my $maxBases     = getGlobal("redBatchLength");
+    fetchStore("unitigging/$asm.ovlStore");
 
-    print STDERR "--\n";
-    print STDERR "-- Configure RED for ", getGlobal("redMemory"), "gb memory.\n";
-    print STDERR "--                   Batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads.\n";
-    print STDERR "--                                      ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
-    print STDERR "--                   Expecting evidence of at most $maxBlockSize bases per iteration.\n";
-    print STDERR "--\n";
-    print STDERR "--           Total                                               Reads                 Olaps Evidence\n";
-    print STDERR "--    Job   Memory      Read Range         Reads        Bases   Memory        Olaps   Memory   Memory  (Memory in MB)\n";
-    print STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
+    #print STDERR "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts\n";
+    open(F, "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts |");
+    while (<F>) {
+        my @v = split '\s+', $_;
+        $numOlaps[$v[0]] = $v[1];
+    }
+    close(F);
 
-    my $reads    = 0;
-    my $bases    = 0;
-    my $olaps    = 0;
+    #  Make an array of partitions, putting as many reads into each as will fit in the desired memory.
 
     my @bgn;
     my @end;
     my $nj = 0;
 
+    #getAllowedResources("", "red");
+
+    my $maxID    = getNumberOfReadsInStore("unitigging", $asm);
+    my $maxMem   = getGlobal("redMemory") * 1024 * 1024 * 1024;
+    my $maxReads = getGlobal("redBatchSize");
+    my $maxBases = getGlobal("redBatchLength");
+
+    print STDERR "\n";
+    print STDERR "Configure RED for ", getGlobal("redMemory"), "gb memory with batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads and ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
+    print STDERR "\n";
+
+    my $reads    = 0;
+    my $bases    = 0;
+    my $olaps    = 0;
+
+    my $coverage = getExpectedCoverage("unitigging", $asm);
+
     push @bgn, 1;
 
     for (my $id = 1; $id <= $maxID; $id++) {
-        if (vec($rlVec, $id, 32) > 0) {
-            $reads += 1;
-            $bases += vec($rlVec, $id, 32);
-            $olaps += vec($noVec, $id, 32);
-        }
+        $reads += 1;
+        $bases += $readLengths[$id];
+        $olaps += $numOlaps[$id];
 
-        #  Memory usage:
-        #
-        #  Per base/vote:
-        #    1 byte  for sequence
-        #   12 bytes for Vote_Tally_t
-        #
-        #  Per read:
-        #   32 bytes for Frag_Info_t
-        #
-        #  Per olap:
-        #   12 bytes for Olap_Info_t
-        #
-        #  When processing, the overlapping reads are loaded in batches of up to 100,000 reads
-        #  (depending on overlaps).  This needs 1 byte per base, but we don't know how many reads
-        #  are getting loaded, so we overestimate by finding the largest block of 100,000 reads that
-        #  could be loaded (done above) and using 2x that (because there are two buffers of these
-        #  reads).
-        #
-        #  Throw in another 2 GB for unknown overheads (gkpStore, ovlStore) and alignment generation.
+        #  Guess how much extra memory used for overlapping reads.  Small genomes tend to load every read in the store,
+        #  large genomes ... load repeats + 2 * coverage * bases in reads (times 2 for overlaps off of each end)
 
-        my $memory = (12 * $bases) + (33 * $reads) + (12 * $olaps) + (2 * $maxBlockSize) + 2 * 1024 * 1024 * 1024;
+        my $memory = (13 * $bases) + (12 * $olaps) + (2 * $bases * $coverage);
 
-        if ((($maxMem   > 0) && ($memory >= $maxMem))    ||
-            (($maxReads > 0) && ($reads  >= $maxReads))  ||
-            (($maxBases > 0) && ($bases  >= $maxBases))  ||
+        if ((($maxMem   > 0) && ($memory >= $maxMem * 0.75)) ||    #  Allow 25% slop (10% is probably sufficient)
+            (($maxReads > 0) && ($reads  >= $maxReads))      ||
+            (($maxBases > 0) && ($bases  >= $maxBases))      ||
             (($id == $maxID))) {
             push @end, $id;
 
-            printf(STDERR "--   %4u %8.2f %9u-%-9u %9u %12u %8.2f %12u %8.2f %8.2f\n",
-                   $nj + 1,
-                   $memory / 1024 / 1024,
-                   $bgn[$nj], $end[$nj],
-                   $reads,
-                   $bases,               (12 * $bases + 33 * $reads)  / 1024 / 1024,
-                   $olaps,               (12 * $olaps)                / 1024 / 1024,
-                   2 * $maxBlockSize / 1024 / 1024);
+            printf(STDERR "RED job %3u from read %9u to read %9u - %7.3f GB for %7u reads - %7.3f GB for %9u olaps - %7.3f GB for evidence\n",
+                   $nj + 1, $bgn[$nj], $end[$nj],
+                   $memory / 1024 / 1024 / 1024, $reads,
+                   13 * $bases / 1024 / 1024 / 1024, $bases,
+                   12 * $olaps / 1024 / 1024 / 1024, $olaps,
+                   2 * $bases * $coverage / 1024 / 1024 / 1024);
 
             $nj++;
 
@@ -233,15 +157,13 @@ sub readErrorDetectionConfigure ($) {
         }
     }
 
-    print  STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
-    printf(STDERR "--                                               %12u          %12u\n",
-           $rlSum, $noSum);
-
     #  Dump a script.
 
     my $batchSize   = getGlobal("redBatchSize");
     my $numThreads  = getGlobal("redThreads");
 
+    my $numReads    = getNumberOfReadsInStore("unitigging", $asm);
+
     open(F, "> $path/red.sh") or caExit("can't open '$path/red.sh' for writing: $!", undef);
 
     print F "#!" . getGlobal("shell") . "\n\n";
@@ -262,7 +184,7 @@ sub readErrorDetectionConfigure ($) {
         print F "fi\n";
     }
 
-    print F "jobid=`printf %05d \$jobid`\n";
+    print F "jobid=`printf %04d \$jobid`\n";
     print F "\n";
     print F "if [ -e ./\$jobid.red ] ; then\n";
     print F "  echo Job previously completed successfully.\n";
@@ -284,11 +206,13 @@ sub readErrorDetectionConfigure ($) {
 
     close(F);
 
-    makeExecutable("$path/red.sh");
+    chmod 0755, "$path/red.sh";
+
     stashFile("$path/red.sh");
 
   finishStage:
     emitStage($asm, "readErrorDetectionConfigure");
+    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -322,7 +246,7 @@ sub readErrorDetectionCheck ($) {
     open(A, "< $path/red.sh") or caExit("can't open '$path/red.sh' for reading: $!", undef);
     while (<A>) {
         if (m/if.*jobid\s+=\s+(\d+)\s+.*then/) {
-            my $ji = substr("00000" . $1, -5);
+            my $ji = substr("0000" . $1, -4);
             my $jn = "unitigging/3-overlapErrorAdjustment/$ji.red";
 
             if (! fileExists($jn)) {
@@ -339,26 +263,25 @@ sub readErrorDetectionCheck ($) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Read error detection jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " read error detection jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Read error detection jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to detect errors in reads.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
         emitStage($asm, "readErrorDetectionCheck", $attempt);
+        buildHTML($asm, "utg");
 
         submitOrRunParallelJob($asm, "red", $path, "red", @failedJobs);
         return;
@@ -402,6 +325,7 @@ sub readErrorDetectionCheck ($) {
     }
 
     emitStage($asm, "readErrorDetectionCheck");
+    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -424,10 +348,27 @@ sub overlapErrorAdjustmentConfigure ($) {
     goto allDone   if (fileExists("unitigging/$asm.ovlStore/evalues"));   #  Stage entrely finished
     goto allDone   if (-d "unitigging/$asm.ctgStore");                    #  Assembly finished
 
-    my $maxID = getNumberOfReadsEarliestVersion($asm);
+    #  OEA uses 1 byte/base + 8 bytes/adjustment + 28 bytes/overlap.  We don't know the number of adjustments, but that's
+    #  basically error rate.  No adjustment is output for mismatches.
 
-    my ($rlVec, $noVec);
-    my ($rlSum, $noSum) = loadReadLengthsAndNumberOfOverlaps($asm, $maxID, \$rlVec, \$noVec);
+    my @readLengths;
+    my @numOlaps;
+
+    #print STDERR "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads\n";
+    open(F, "$bin/gatekeeperDumpMetaData -G unitigging/$asm.gkpStore -reads |");
+    while (<F>) {
+        my @v = split '\s+', $_;
+        $readLengths[$v[0]] = $v[2];
+    }
+    close(F);
+
+    #print STDERR "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts\n";
+    open(F, "$bin/ovStoreDump -G unitigging/$asm.gkpStore -O unitigging/$asm.ovlStore -d -counts |");
+    while (<F>) {
+        my @v = split '\s+', $_;
+        $numOlaps[$v[0]] = $v[1];
+    }
+    close(F);
 
     #  Make an array of partitions, putting as many reads into each as will fit in the desired memory.
 
@@ -438,15 +379,13 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     my $nj = 0;
 
+    my $maxID    = getNumberOfReadsInStore("unitigging", $asm);
     my $maxMem   = getGlobal("oeaMemory") * 1024 * 1024 * 1024;
     my $maxReads = getGlobal("oeaBatchSize");
     my $maxBases = getGlobal("oeaBatchLength");
 
-    print STDERR "--\n";
-    print STDERR "-- Configure OEA for ", getGlobal("oeaMemory"), "gb memory.\n";
-    print STDERR "--                   Batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads.\n";
-    print STDERR "--                                      ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
-    print STDERR "--\n";
+    print STDERR "\n";
+    print STDERR "Configure OEA for ", getGlobal("oeaMemory"), "gb memory with batches of at most ", ($maxReads > 0) ? $maxReads : "(unlimited)", " reads and ", ($maxBases > 0) ? $maxBases : "(unlimited)", " bases.\n";
 
     my $reads    = 0;
     my $bases    = 0;
@@ -454,6 +393,7 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     fetchFile("$path/red.red");
 
+    my $coverage     = getExpectedCoverage("unitigging", $asm);
     my $corrSize     = (-s "$path/red.red");
 
     my $smallJobs    = 0;
@@ -462,48 +402,42 @@ sub overlapErrorAdjustmentConfigure ($) {
     push @bgn, 1;
 
     for (my $id = 1; $id <= $maxID; $id++) {
-        if (vec($rlVec, $id, 32) > 0) {
-            $reads += 1;
-            $bases += vec($rlVec, $id, 32);
-            $olaps += vec($noVec, $id, 32);
-        }
-
-        #  OEA uses 1 byte/base + 8 bytes/adjustment + 28 bytes/overlap.  We don't know the number
-        #  of adjustments, but that's basically error rate.  No adjustment is output for mismatches.
+        $reads += 1;
+        $bases += $readLengths[$id];
+        $olaps += $numOlaps[$id];
 
         #  Hacked to attempt to estimate adjustment size better.  Olaps should only require 12 bytes each.
 
-        my $memBases  = (1    * $bases);              #  Corrected reads for this batch
-        my $memAdj1   = (8    * $corrSize) * 0.33;    #  Overestimate of the size of the indel adjustments needed (total size includes mismatches)
-        my $memReads  = (32   * $reads);              #  Read data in the batch
-        my $memOlaps  = (32   * $olaps);              #  Loaded overlaps
-        my $memSeq    = (4    * 2097152);             #  two char arrays of 2*maxReadLen
-        my $memAdj2   = (16   * 2097152);             #  two Adjust_t arrays of maxReadLen
-        my $memWA     = (32   * 1048576);             #  Work area (16mb) and edit array (16mb)
-        my $memMisc   = (256  * 1048576);             #  Work area (16mb) and edit array (16mb) and (192mb) slop
-        my $memExtra  = (2048 * 1048576);             #  For alignments and overhead.
+        my $memBases  = (1   * $bases);              #  Corrected reads for this batch
+        my $memAdj1   = (8   * $corrSize) * 0.33;    #  Overestimate of the size of the indel adjustments needed (total size includes mismatches)
+        my $memReads  = (32  * $reads);              #  Read data in the batch
+        my $memOlaps  = (32  * $olaps);              #  Loaded overlaps
+        my $memSeq    = (4   * 2097152);             #  two char arrays of 2*maxReadLen
+        my $memAdj2   = (16  * 2097152);             #  two Adjust_t arrays of maxReadLen
+        my $memWA     = (32  * 1048576);             #  Work area (16mb) and edit array (16mb)
+        my $memMisc   = (256 * 1048576);             #  Work area (16mb) and edit array (16mb) and (192mb) slop
 
-        my $memory = $memBases + $memAdj1 + $memReads + $memOlaps + $memSeq + $memAdj2 + $memWA + $memMisc + $memExtra;
+        my $memory = $memBases + $memAdj1 + $memReads + $memOlaps + $memSeq + $memAdj2 + $memWA + $memMisc;
 
-        if ((($maxMem   > 0) && ($memory >= $maxMem))   ||
-            (($maxReads > 0) && ($reads  >= $maxReads)) ||
-            (($maxBases > 0) && ($bases  >= $maxBases)) ||
+        if ((($maxMem   > 0) && ($memory >= $maxMem * 0.75)) ||
+            (($maxReads > 0) && ($reads  >= $maxReads))      ||
+            (($maxBases > 0) && ($bases  >= $maxBases))      ||
             (($id == $maxID))) {
             push @end, $id;
 
             $smallJobs++   if ($end[$nj] - $bgn[$nj] < $smallJobSize);
 
-            #  Save the log for later printing.  We redo the configuration if there are too many small jobs.
-
-            push @log, sprintf("--   %4u %8.2f %9u-%-9u %9u %12u %8.2f %12u %8.2f %8.2f\n",
-                               $nj + 1,
-                               $memory / 1024 / 1024,
-                               $bgn[$nj], $end[$nj],
-                               $reads, $bases,
-                               ($memReads + $memBases + $memSeq) / 1024 / 1024,
-                               $olaps,
-                               $memOlaps / 1024 / 1024,
-                               ($memAdj1 + $memAdj2 + $memWA + $memMisc) / 1024 / 1024);
+            push @log, sprintf("OEA job %3u from read %9u to read %9u - %4.1f bases + %4.1f adjusts + %4.1f reads + %4.1f olaps + %4.1f fseq/rseq + %4.1f fadj/radj + %4.1f work + %4.1f misc = %5.1f MB\n",
+                   $nj + 1, $bgn[$nj], $end[$nj],
+                   $memBases / 1024 / 1024,
+                   $memAdj1  / 1024 / 1024,
+                   $memReads / 1024 / 1024,
+                   $memOlaps / 1024 / 1024,
+                   $memSeq   / 1024 / 1024,
+                   $memAdj2  / 1024 / 1024,
+                   $memWA    / 1024 / 1024,
+                   $memMisc  / 1024 / 1024,
+                   $memory   / 1024 / 1024);
 
             $nj++;
 
@@ -531,17 +465,14 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     #  Report.
 
-    print STDERR "--           Total                                               Reads                 Olaps  Adjusts\n";
-    print STDERR "--    Job   Memory      Read Range         Reads        Bases   Memory        Olaps   Memory   Memory  (Memory in MB)\n";
-    print STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
+    print STDERR "Configured $nj jobs.\n";
+    print STDERR "\n";
 
     foreach my $l (@log) {
         print STDERR $l;
     }
 
-    print  STDERR "--   ---- -------- ------------------- --------- ------------ -------- ------------ -------- --------\n";
-    printf(STDERR "--                                               %12u          %12u\n",
-           $rlSum, $noSum);
+    print STDERR "\n";
 
     #  Dump a script
 
@@ -565,7 +496,7 @@ sub overlapErrorAdjustmentConfigure ($) {
         print F "fi\n";
     }
 
-    print F "jobid=`printf %05d \$jobid`\n";
+    print F "jobid=`printf %04d \$jobid`\n";
     print F "\n";
     print F "if [ -e ./\$jobid.oea ] ; then\n";
     print F "  echo Job previously completed successfully.\n";
@@ -589,11 +520,13 @@ sub overlapErrorAdjustmentConfigure ($) {
 
     close(F);
 
-    makeExecutable("$path/oea.sh");
+    chmod 0755, "$path/oea.sh";
+
     stashFile("$path/oea.sh");
 
   finishStage:
     emitStage($asm, "overlapErrorAdjustmentConfigure");
+    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -620,7 +553,8 @@ sub overlapErrorAdjustmentCheck ($) {
 
     my $batchSize   = getGlobal("oeaBatchSize");
     my $failedJobs  = 0;
-    my $numJobs     = 0;
+    my $numReads    = getNumberOfReadsInStore("unitigging", $asm);
+    my $numJobs     = 0;  #int($numReads / $batchSize) + (($numReads % $batchSize == 0) ? 0 : 1);
 
     #  Need to read script to find number of jobs!
 
@@ -633,7 +567,7 @@ sub overlapErrorAdjustmentCheck ($) {
     open(A, "< $path/oea.sh") or caExit("can't open '$path/oea.sh' for reading: $!", undef);
     while (<A>) {
         if (m/if.*jobid\s+=\s+(\d+)\s+.*then/) {
-            my $ji = substr("00000" . $1, -5);
+            my $ji = substr("0000" . $1, -4);
 
             if (! fileExists("unitigging/3-overlapErrorAdjustment/$ji.oea")) {
                 $failureMessage .= "--   job $ji.oea FAILED.\n";
@@ -649,26 +583,25 @@ sub overlapErrorAdjustmentCheck ($) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Overlap error adjustment jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " overlap error adjustment jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Overlap error adjustment jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to adjust overlap error rates.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
         emitStage($asm, "overlapErrorAdjustmentCheck", $attempt);
+        buildHTML($asm, "utg");
 
         submitOrRunParallelJob($asm, "oea", $path, "oea", @failedJobs);
         return;
@@ -686,6 +619,7 @@ sub overlapErrorAdjustmentCheck ($) {
     stashFile("$path/oea.files");
 
     emitStage($asm, "overlapErrorAdjustmentCheck");
+    buildHTML($asm, "utg");
 
   allDone:
 }
@@ -744,6 +678,7 @@ sub updateOverlapStore ($) {
 
   finishStage:
     emitStage($asm, "updateOverlapStore");
+    buildHTML($asm, "utg");
 
   allDone:
 }
