@@ -49,6 +49,7 @@ use File::Path 2.08 qw(make_path remove_tree);
 use canu::Defaults;
 use canu::Execution;
 use canu::Report;
+use canu::HTML;
 use canu::Grid_Cloud;
 
 
@@ -105,15 +106,30 @@ sub overlapConfigure ($$$) {
         #  Salmon assembly when overlaps were computed differently depending on the libraries
         #  involved (and was run manually).  These are left in for documentation.
         #
-        #my $checkLibrary       = getGlobal("${tag}CheckLibrary");    #  -C
-        #my $hashLibrary        = getGlobal("${tag}HashLibrary");     #  -H $hashLibrary
-        #my $refLibrary         = getGlobal("${tag}RefLibrary");      #  -R $refLibrary
+        #my $checkLibrary       = getGlobal("${tag}CheckLibrary");
+        #my $hashLibrary        = getGlobal("${tag}HashLibrary");
+        #my $refLibrary         = getGlobal("${tag}RefLibrary");
+
+        my $hashBlockLength = getGlobal("${tag}OvlHashBlockLength");
+        my $hashBlockSize   = 0;
+        my $refBlockSize    = getGlobal("${tag}OvlRefBlockSize");
+        my $refBlockLength  = getGlobal("${tag}OvlRefBlockLength");
+        my $minOlapLength   = getGlobal("minOverlapLength");
+
+        if (($refBlockSize > 0) && ($refBlockLength > 0)) {
+            caExit("can't set both ${tag}OvlRefBlockSize and ${tag}OvlRefBlockLength", undef);
+        }
 
         $cmd  = "$bin/overlapInCorePartition \\\n";
-        $cmd .= " -G  ../$asm.gkpStore \\\n";
-        $cmd .= " -hl " . getGlobal("${tag}OvlHashBlockLength") . " \\\n";
-        $cmd .= " -rl " . getGlobal("${tag}OvlRefBlockLength")  . " \\\n";
-        $cmd .= " -ol " . getGlobal("minOverlapLength") . " \\\n";
+        $cmd .= " -g  ../$asm.gkpStore \\\n";
+        $cmd .= " -bl $hashBlockLength \\\n";
+        $cmd .= " -bs $hashBlockSize \\\n";
+        $cmd .= " -rs $refBlockSize \\\n";
+        $cmd .= " -rl $refBlockLength \\\n";
+        #$cmd .= " -H $hashLibrary \\\n" if ($hashLibrary ne "0");
+        #$cmd .= " -R $refLibrary \\\n"  if ($refLibrary ne "0");
+        #$cmd .= " -C \\\n" if (!$checkLibrary);
+        $cmd .= " -ol $minOlapLength \\\n";
         $cmd .= " -o  ./$asm.partition \\\n";
         $cmd .= "> ./$asm.partition.err 2>&1";
 
@@ -157,6 +173,8 @@ sub overlapConfigure ($$$) {
         open(F, "> $path/overlap.sh") or caExit("can't open '$path/overlap.sh' for writing: $!", undef);
         print F "#!" . getGlobal("shell") . "\n";
         print F "\n";
+        print F "perl='/usr/bin/env perl'\n";
+        print F "\n";
         print F getBinDirectoryShellCode();
         print F "\n";
         print F setWorkDirectoryShellCode($path);
@@ -214,7 +232,8 @@ sub overlapConfigure ($$$) {
         print F "exit 0\n";
         close(F);
 
-        makeExecutable("$path/overlap.sh");
+        system("chmod +x $path/overlap.sh");
+
         stashFile("$path/overlap.sh");
     }
 
@@ -233,6 +252,7 @@ sub overlapConfigure ($$$) {
 
   finishStage:
     emitStage($asm, "$tag-overlapConfigure");
+    buildHTML($asm, $tag);
 
   allDone:
     stopAfter("overlapConfigure");
@@ -371,7 +391,7 @@ sub overlapCheck ($$$) {
                 push @miscJobs,    "1-overlapper/$1.counts\n";
 
             } else {
-                $failureMessage .= "--   job $path/$1.ovb FAILED.\n";
+                $failureMessage .= "--   job 1-overlapper/$1.ovb FAILED.\n";
                 push @failedJobs, $currentJobID;
             }
 
@@ -385,26 +405,25 @@ sub overlapCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Overlap jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " overlapper jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Overlap jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to overlap.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
         emitStage($asm, "$tag-overlapCheck", $attempt);
+        buildHTML($asm, $tag);
 
         submitOrRunParallelJob($asm, "${tag}ovl", $path, "overlap", @failedJobs);
         return;
@@ -427,6 +446,7 @@ sub overlapCheck ($$$) {
     reportOverlapStats($base, $asm, @statsJobs);
 
     emitStage($asm, "$tag-overlapCheck");
+    buildHTML($asm, $tag);
 
   allDone:
     stopAfter("overlap");
