@@ -44,13 +44,12 @@ require Exporter;
 
 use strict;
 use File::Basename;   #  dirname
-use File::Path 2.08 qw(make_path remove_tree);
 
 use POSIX qw(ceil);
 use canu::Defaults;
 use canu::Execution;
-use canu::Gatekeeper;
 use canu::Report;
+use canu::HTML;
 use canu::Grid_Cloud;
 
 
@@ -204,13 +203,14 @@ sub overlapStoreConfigure ($$$) {
         open(F, "> $path/scripts/0-config.sh") or die;
         print F "#!" . getGlobal("shell") . "\n";
         print F "\n";
-        print F getBinDirectoryShellCode();
+        print F setWorkDirectoryShellCode($path);
         print F "\n";
-        #print F setWorkDirectoryShellCode($path);   #  This is never run on grid, so don't need to cd first.
-        #print F "\n";
         #print F getJobIDShellCode();
         #print F "\n";
-        print F getLimitShellCode();
+        print F getBinDirectoryShellCode();
+        print F "\n";
+        print F getLimitShellCode("processes");
+        print F getLimitShellCode("files");
         print F "\n";
         print F "\$bin/ovStoreBuild \\\n";
         print F " -G ./$asm.gkpStore \\\n";
@@ -220,8 +220,7 @@ sub overlapStoreConfigure ($$$) {
         print F " -L ./1-overlapper/ovljob.files \\\n";
         close(F);
     }
-    makeExecutable("$path/scripts/0-config.sh");
-    stashFile("$path/scripts/0-config.sh");
+    system("chmod +x $path/scripts/0-config.sh");
 
     if (! -e "$path/config") {
         $cmd  = "./$asm.ovlStore.BUILDING/scripts/0-config.sh \\\n";
@@ -238,16 +237,18 @@ sub overlapStoreConfigure ($$$) {
 
     #  Parallel jobs for bucketizing.  This should really be part of overlap computation itself.
 
+    #getAllowedResources("", "ovb");
+
     if (! -e "$path/scripts/1-bucketize.sh") {
         open(F, "> $path/scripts/1-bucketize.sh") or die;
         print F "#!" . getGlobal("shell") . "\n";
         print F "\n";
-        print F getBinDirectoryShellCode();
-        print F "\n";
         print F setWorkDirectoryShellCode($path);
         print F "\n";
         print F getJobIDShellCode();
         print F "\n";
+        print F getBinDirectoryShellCode();
+        print F "\n";
         print F "bn=`printf %04d \$jobid`\n";
         print F "jn=\"undefined\"\n";
         print F "\n";
@@ -284,7 +285,8 @@ sub overlapStoreConfigure ($$$) {
         print F "  rm -rf \"./create\$bn\"\n";
         print F "fi\n";
         print F "\n";
-        print F getLimitShellCode();
+        print F getLimitShellCode("processes");
+        print F getLimitShellCode("files");
         print F "\n";
         print F "\$bin/ovStoreBucketizer \\\n";
         print F "  -O . \\\n";
@@ -293,22 +295,33 @@ sub overlapStoreConfigure ($$$) {
         #print F "  -e " . getGlobal("") . " \\\n"  if (defined(getGlobal("")));
         print F "  -job \$jobid \\\n";
         print F "  -i   \$jn\n";
+        print F "\n";
+        print F "if [ \$? = 0 ] ; then\n";
+        print F "  echo Success.\n";
+        print F "  exit 0\n";
+        print F "else\n";
+        print F "  echo Failure.\n";
+        print F "  exit 1\n";
+        print F "fi\n";
         close(F);
     }
 
     #  Parallel jobs for sorting each bucket
 
+    #getAllowedResources("", "ovs");
+
     if (! -e "$path/scripts/2-sort.sh") {
         open(F, "> $path/scripts/2-sort.sh") or die;
         print F "#!" . getGlobal("shell") . "\n";
         print F "\n";
-        print F getBinDirectoryShellCode();
-        print F "\n";
         print F setWorkDirectoryShellCode($path);
         print F "\n";
         print F getJobIDShellCode();
         print F "\n";
-        print F getLimitShellCode();
+        print F getBinDirectoryShellCode();
+        print F "\n";
+        print F getLimitShellCode("processes");
+        print F getLimitShellCode("files");
         print F "\n";
         print F "\$bin/ovStoreSorter \\\n";
         print F "  -deletelate \\\n";  #  Choices -deleteearly -deletelate or nothing
@@ -317,6 +330,14 @@ sub overlapStoreConfigure ($$$) {
         print F "  -G ../$asm.gkpStore \\\n";
         print F "  -F $numSlices \\\n";
         print F "  -job \$jobid $numInputs\n";
+        print F "\n";
+        print F "if [ \$? = 0 ] ; then\n";
+        print F "  echo Success.\n";
+        print F "  exit 0\n";
+        print F "else\n";
+        print F "  echo Failure.\n";
+        print F "  exit 1\n";
+        print F "fi\n";
         close(F);
     }
 
@@ -326,29 +347,34 @@ sub overlapStoreConfigure ($$$) {
         open(F, "> $path/scripts/3-index.sh") or die;
         print F "#!" . getGlobal("shell") . "\n";
         print F "\n";
-        print F getBinDirectoryShellCode();
+        print F setWorkDirectoryShellCode($path);
         print F "\n";
-        #print F setWorkDirectoryShellCode($path);   #  This is never run on grid, so don't need to cd first.
-        #print F "\n";
         #print F getJobIDShellCode();
         #print F "\n";
+        print F getBinDirectoryShellCode();
+        print F "\n";
         print F "\$bin/ovStoreIndexer \\\n";
         #print F "  -nodelete \\\n";  #  Choices -nodelete or nothing
         print F "  -O . \\\n";
         print F "  -F $numSlices\n";
+        print F "\n";
+        print F "if [ \$? = 0 ] ; then\n";
+        print F "  echo Success.\n";
+        print F "  exit 0\n";
+        print F "else\n";
+        print F "  echo Failure.\n";
+        print F "  exit 1\n";
+        print F "fi\n";
         close(F);
     }
 
-    makeExecutable("$path/scripts/1-bucketize.sh");
-    makeExecutable("$path/scripts/2-sort.sh");
-    makeExecutable("$path/scripts/3-index.sh");
-
-    stashFile("$path/scripts/1-bucketize.sh");
-    stashFile("$path/scripts/2-sort.sh");
-    stashFile("$path/scripts/3-index.sh");
+    system("chmod +x $path/scripts/1-bucketize.sh");
+    system("chmod +x $path/scripts/2-sort.sh");
+    system("chmod +x $path/scripts/3-index.sh");
 
   finishStage:
     emitStage($asm, "$tag-overlapStoreConfigure");
+    buildHTML($asm, $tag);
 
   allDone:
     stopAfter("overlapStoreConfigure");
@@ -405,26 +431,25 @@ sub overlapStoreBucketizerCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Overlap store bucketizer jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " overlap store bucketizer jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Overlap store bucketizer jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to overlapStoreBucketize.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
         emitStage($asm, "$tag-overlapStoreBucketizerCheck", $attempt);
+        buildHTML($asm, $tag);
 
         submitOrRunParallelJob($asm, "ovB", $path, "scripts/1-bucketize", @failedJobs);
         return;
@@ -436,6 +461,7 @@ sub overlapStoreBucketizerCheck ($$$) {
     touch("$path/1-bucketize.success");
 
     emitStage($asm, "$tag-overlapStoreBucketizerCheck");
+    buildHTML($asm, $tag);
 
   allDone:
 }
@@ -502,27 +528,25 @@ sub overlapStoreSorterCheck ($$$) {
 
     if (scalar(@failedJobs) > 0) {
 
-        #  If too many attempts, give up.
+        #  If not the first attempt, report the jobs that failed, and that we're recomputing.
 
-
-        if ($attempt >= getGlobal("canuIterationMax")) {
+        if ($attempt > 1) {
             print STDERR "--\n";
-            print STDERR "-- Overlap store sorting jobs failed, tried $attempt times, giving up.\n";
+            print STDERR "-- ", scalar(@failedJobs), " overlap store sorter jobs failed:\n";
             print STDERR $failureMessage;
             print STDERR "--\n";
-            caExit(undef, undef);
         }
 
-        if ($attempt > 0) {
-            print STDERR "--\n";
-            print STDERR "-- Overlap store sorting jobs failed, retry.\n";
-            print STDERR $failureMessage;
-            print STDERR "--\n";
+        #  If too many attempts, give up.
+
+        if ($attempt > getGlobal("canuIterationMax")) {
+            caExit("failed to overlapStoreSorter.  Made " . ($attempt-1) . " attempts, jobs still failed", undef);
         }
 
         #  Otherwise, run some jobs.
 
         emitStage($asm, "$tag-overlapStoreSorterCheck", $attempt);
+        buildHTML($asm, $tag);
 
         submitOrRunParallelJob($asm, "ovS", $path, "scripts/2-sort", @failedJobs);
         return;
@@ -534,12 +558,14 @@ sub overlapStoreSorterCheck ($$$) {
     touch("$path/2-sorter.success");
 
     emitStage($asm, "$tag-overlapStoreSorterCheck");
+    buildHTML($asm, $tag);
 
   allDone:
 }
 
 
 
+
 sub createOverlapStoreParallel ($$$) {
     my $base    = shift @_;
     my $asm     = shift @_;
@@ -558,51 +584,41 @@ sub createOverlapStoreParallel ($$$) {
 }
 
 
-
-sub checkOverlapStore ($$) {
+sub generateOverlapStoreStats ($$) {
     my $base    = shift @_;
     my $asm     = shift @_;
 
     my $bin   = getBinDirectory();
     my $cmd;
 
-    $cmd  = "$bin/ovStoreDump \\\n";
+    $cmd  = "$bin/ovStoreStats \\\n";
     $cmd .= " -G ./$asm.gkpStore \\\n";
     $cmd .= " -O ./$asm.ovlStore \\\n";
-    $cmd .= " -d -counts \\\n";
-    $cmd .= " > ./$asm.ovlStore/counts.dat 2> ./$asm.ovlStore/counts.err";
-
-    print STDERR "-- Checking store.\n";
+    $cmd .= " -o ./$asm.ovlStore \\\n";
+    $cmd .= " > ./$asm.ovlStore.summary.err 2>&1";
 
     if (runCommand($base, $cmd)) {
-        caExit("failed to dump counts of overlaps; invalid store?", "./$asm.ovlStore/counts.err");
+        print STDERR "--\n";
+        print STDERR "-- WARNING: failed to generate statistics for the overlap store; no summary will appear in report.\n";
+        print STDERR "--\n";
+        print STDERR "----------------------------------------\n";
+        return;
     }
 
-    my $totOvl   = 0;
-    my $nulReads = 0;
-    my $ovlReads = 0;
+    unlink "$base/$asm.ovlStore.summary.err";
 
-    open(F, "< ./$base/$asm.ovlStore/counts.dat") or die;
+    my $report;
+
+    open(F, "< $base/$asm.ovlStore.summary") or caExit("Failed to open overlap store statistics in '$base/$asm.ovlStore.summary': $!", undef);
     while (<F>) {
-        my @v = split '\s+', $_;
-
-        $nulReads += 1       if ($v[1] < 1);
-        $ovlReads += 1       if ($v[1] > 0);
-        $totOvl   += $v[1];
+        $report .= "-- $_";
     }
     close(F);
 
-    print STDERR "--\n";
-    print STDERR "-- Overlap store '$base/$asm.ovlStore' successfully constructed.\n";
-    print STDERR "-- Found $totOvl overlaps for $ovlReads reads; $nulReads reads have no overlaps.\n";
-    print STDERR "--\n";
-
-    unlink "./$base/$asm.ovlStore/counts.dat";
-    unlink "./$base/$asm.ovlStore/counts.err";
+    addToReport("overlaps", $report);
 }
 
 
-
 sub createOverlapStore ($$$) {
     my $asm     = shift @_;
     my $tag     = shift @_;
@@ -627,24 +643,13 @@ sub createOverlapStore ($$$) {
     createOverlapStoreSequential($base, $asm, $tag)  if ($seq eq "sequential");
     createOverlapStoreParallel  ($base, $asm, $tag)  if ($seq eq "parallel");
 
-    checkOverlapStore($base, $asm);
+    print STDERR "--\n";
+    print STDERR "-- Overlap store '$base/$asm.ovlStore' successfully constructed.\n";
 
     goto finishStage  if (getGlobal("saveOverlaps") eq "1");
 
-    #  Delete the inputs and directories.
-    #
-    #    Directories - Viciously remove the whole thing (after all files are deleted, so we
-    #                  can get the sizes).
-    #    Files       - Sum the size, remove the file, and try to remove the directory.  In
-    #                  particular, we don't want to remove_tree() this directory - there could
-    #                  be other stuff in it - only remove if empty.
-    #
-    #  Ideally, every directory we have in our list should be empty after we delete the files in the
-    #  list.  But they won't be.  Usually because there are empty directories in there too.  Maybe
-    #  some stray files we didn't track.  Regardless, just blow them away.
-    #
-    #  Previous (to July 2017) versions tried to gently rmdir things, but it was ugly and didn't
-    #  quite work.
+    #  Delete the inputs and directories.  Some contortions are needed to get directory deletes in order.
+    #  In particular, mhap's queries directory needs to be deleted after it's subdirectories are.
 
     my %directories;
     my $bytes = 0;
@@ -661,76 +666,76 @@ sub createOverlapStore ($$$) {
         while (<F>) {
             chomp;
 
+            #  Decide what to do.  Directories - register for later deletion.  Files - sum size and
+            #  delete.
+
             if (-d "$base/$_") {
-                $directories{$_}++;
+                $directories{"$base/$_"}++;
 
             } elsif (-e "$base/$_") {
                 $bytes += -s "$base/$_";
                 $files += 1;
 
                 unlink "$base/$_";
-                rmdir dirname("$base/$_");  #  Try to rmdir the directory the file is in.  If empty, yay!
             }
+
+            #  If the path isn't a directory register the directory it is in for deletion.
+
+            $directories{dirname("$base/$_")}++   if (! -d "$base/$_");
         }
         close(F);
+
+        unlink $file;
     }
 
-    foreach my $dir (keys %directories) {
-        remove_tree("$base/$dir");
-    }
+    #  Ideally, every directory we have in our list should be empty.  But they won't be.  So, loop until we fail to delete anything.
 
-    unlink "$base/1-overlapper/ovljob.files";
-    unlink "$base/1-overlapper/ovljob.more.files";
-    unlink "$base/1-overlapper/mhap.files";
-    unlink "$base/1-overlapper/mmap.files";
-    unlink "$base/1-overlapper/precompute.files";
+    my $dirs = 0;
+    my $deleted = 1;
+
+    while ($deleted > 0) {
+        $deleted = 0;
+
+        foreach my $dir (keys %directories) {
+            if (-d $dir) {
+                rmdir $dir;
+                $dirs++;
+            }
+
+            if (! -d $dir) {  #  If really removed, remove it from our list.
+                delete $directories{$dir};
+                $deleted++;
+            }
+        }
+    }
 
     print STDERR "--\n";
-    print STDERR "-- Purged ", int(1000 * $bytes / 1024 / 1024 / 1024) / 1000, " GB in $files overlap output files.\n";
+    print STDERR "-- Purged ", int(1000 * $bytes / 1024 / 1024 / 1024) / 1000, " GB in $files overlap output files and $dirs directories.\n";
 
     #  Now all done!
 
   finishStage:
     if ($tag eq "utg") {
-        my $bin   = getBinDirectory();
-        my $cmd;
+        generateOverlapStoreStats($base, $asm);
+    }
 
-        $cmd  = "$bin/ovStoreStats \\\n";
-        $cmd .= " -G ./$asm.gkpStore \\\n";
-        $cmd .= " -O ./$asm.ovlStore \\\n";
-        $cmd .= " -C " . getExpectedCoverage("utg", $asm). " \\\n";
-        $cmd .= " -o ./$asm.ovlStore \\\n";
-        $cmd .= " > ./$asm.ovlStore.summary.err 2>&1";
+    if (-e "$base/$asm.ovlStore.summary") {
+        print STDERR "--\n";
+        print STDERR "-- Overlap store '$base/$asm.ovlStore' contains:\n";
+        print STDERR "--\n";
 
-        if (runCommand($base, $cmd)) {
-            #  Do nothing if it fails.  This is just logging.
+        open(F, "< $base/$asm.ovlStore.summary") or caExit("Failed to open overlap store statistics in '$base/$asm.ovlStore': $!", undef);
+        while (<F>) {
+            print STDERR "--   $_";
         }
+        close(F);
 
-        unlink "$base/$asm.ovlStore.summary.err";
-
-        if (! -e "$base/$asm.ovlStore.summary") {
-            print STDERR "--\n";
-            print STDERR "-- WARNING: failed to generate statistics for the overlap store; no summary will appear in report.\n";
-            print STDERR "--\n";
-            print STDERR "----------------------------------------\n";
-        } else {
-            print STDERR "--\n";
-            print STDERR "-- Overlap store '$base/$asm.ovlStore' contains:\n";
-            print STDERR "--\n";
-
-            my $report;
-
-            open(F, "< $base/$asm.ovlStore.summary") or caExit("Failed to open overlap store statistics in '$base/$asm.ovlStore': $!", undef);
-            while (<F>) {
-                $report .= "--   $_";
-            }
-            close(F);
-
-            addToReport("overlaps", $report);   #  Also shows it.
-        }
+    } else {
+        print STDERR "-- Overlap store '$base/$asm.ovlStore' statistics not available (skipped in correction and trimming stages).\n";
     }
 
     emitStage($asm, "$tag-createOverlapStore");
+    buildHTML($asm, $tag);
 
   allDone:
     stopAfter("overlapStore");
